@@ -16,11 +16,10 @@ import math
 from collections.abc import Iterator
 from typing import Callable, List, Optional, Union
 
-import numpy as np
 import torch
 from omegaconf import DictConfig
 
-from nemo.collections.asr.parts import parsers
+from nemo.collections.common.parts.preprocessing import parsers
 from nemo.utils.decorators import experimental
 
 try:
@@ -128,6 +127,7 @@ class AudioToCharDALIDataset(Iterator):
         world_size: int = 1,
         preprocessor_cfg: DictConfig = None,
     ):
+        self.drop_last = drop_last  # used by lr_scheduler
         if not HAVE_DALI:
             raise ModuleNotFoundError(
                 f"{self} requires NVIDIA DALI to be installed. "
@@ -173,18 +173,18 @@ class AudioToCharDALIDataset(Iterator):
 
         has_preprocessor = preprocessor_cfg is not None
         if has_preprocessor:
-            if preprocessor_cfg.cls == "nemo.collections.asr.modules.AudioToMelSpectrogramPreprocessor":
+            if preprocessor_cfg._target_ == "nemo.collections.asr.modules.AudioToMelSpectrogramPreprocessor":
                 feature_type = "mel_spectrogram"
-            elif preprocessor_cfg.cls == "nemo.collections.asr.modules.AudioToMFCCPreprocessor":
+            elif preprocessor_cfg._target_ == "nemo.collections.asr.modules.AudioToMFCCPreprocessor":
                 feature_type = "mfcc"
             else:
                 raise ValueError(
-                    f"{self} received an unexpected preprocessor configuration: {preprocessor_cfg.cls}."
+                    f"{self} received an unexpected preprocessor configuration: {preprocessor_cfg._target_}."
                     f" Supported preprocessors are: AudioToMelSpectrogramPreprocessor, AudioToMFCCPreprocessor"
                 )
 
             # Default values taken from AudioToMelSpectrogramPreprocessor
-            params = preprocessor_cfg.params
+            params = preprocessor_cfg
             self.dither = params['dither'] if 'dither' in params else 0.0
             self.preemph = params['preemph'] if 'preemph' in params else 0.97
             self.window_size_sec = params['window_size'] if 'window_size' in params else 0.02
@@ -291,7 +291,7 @@ class AudioToCharDALIDataset(Iterator):
                 random_shuffle=shuffle,
                 shard_id=self.shard_id,
                 num_shards=self.num_shards,
-                pad_last_batch=True,
+                pad_last_batch=False,
             )
 
             transcript_len = dali.fn.shapes(dali.fn.reshape(transcript, shape=[-1]))
@@ -310,8 +310,8 @@ class AudioToCharDALIDataset(Iterator):
 
             if not has_preprocessor:
                 # No preprocessing, the output is the audio signal
-                audio = dali.fn.pad(audio)
                 audio_len = dali.fn.shapes(dali.fn.reshape(audio, shape=[-1]))
+                audio = dali.fn.pad(audio)
                 self.pipe.set_outputs(audio, audio_len, transcript, transcript_len)
             else:
                 # Additive gaussian noise (dither)
@@ -354,20 +354,11 @@ class AudioToCharDALIDataset(Iterator):
                 spec = dali.fn.normalize(spec, axes=self.normalization_axes)
 
                 # Extracting the length of the spectrogram
-                shape_start = dali.types.Constant(np.array([1], dtype=np.float32), device='cpu')
-                shape_len = dali.types.Constant(np.array([1], dtype=np.float32), device='cpu')
-                spec_len = dali.fn.slice(
-                    dali.fn.shapes(spec),
-                    shape_start,
-                    shape_len,
-                    normalized_anchor=False,
-                    normalized_shape=False,
-                    axes=(0,),
-                )
+                spec_len = dali.fn.slice(dali.fn.shapes(spec), 1, 1, axes=(0,))
 
                 # Pads feature dimension to be a multiple of `pad_to` and the temporal dimension to be as big as the largest sample (shape -1)
                 spec = dali.fn.pad(spec, fill_value=self.pad_value, axes=(0, 1), align=(self.pad_to, 1), shape=(1, -1))
-            self.pipe.set_outputs(spec, spec_len, transcript, transcript_len)
+                self.pipe.set_outputs(spec, spec_len, transcript, transcript_len)
         # Building DALI pipeline
         self.pipe.build()
 
@@ -415,9 +406,15 @@ class AudioToCharDALIDataset(Iterator):
     def __next__(self):
         outputs = self._iter.next()
         assert len(outputs) == 1
-        out = outputs[0]
-        text_raw_len = out['transcript_raw_len'].numpy()
-        text_raw = out['transcript_raw'].numpy()
+        dali_out = outputs[0]
+        text_raw_len = dali_out['transcript_raw_len'].numpy()
+        text_raw = dali_out['transcript_raw'].numpy()
+
+        out = {}
+        out_names = ['processed_signal', 'processed_signal_len', 'audio', 'audio_len']
+        for out_name in out_names:
+            if out_name in dali_out:
+                out[out_name] = dali_out[out_name].detach().clone()
 
         text_tokens = []
         text_tokens_len = []
