@@ -13,7 +13,7 @@
 # limitations under the License.
 from collections import OrderedDict
 from dataclasses import dataclass, field
-from typing import List, Optional
+from typing import List, Optional, Union
 
 import torch
 import torch.nn as nn
@@ -23,10 +23,16 @@ from omegaconf import MISSING, ListConfig, OmegaConf
 from nemo.collections.asr.parts.submodules.jasper import (
     JasperBlock,
     MaskedConv1d,
-    StatsPoolLayer,
+    ParallelBlock,
     init_weights,
     jasper_activations,
 )
+from nemo.collections.asr.parts.submodules.tdnn_attention import (
+    AttentivePoolLayer,
+    StatsPoolLayer,
+    TDNNModule,
+    TDNNSEModule,
+)
 from nemo.core.classes.common import typecheck
 from nemo.core.classes.exportable import Exportable
 from nemo.core.classes.module import NeuralModule
@@ -201,6 +207,172 @@ class ConvASREncoder(NeuralModule, Exportable):
         return s_input[-1], length
 
 
+class ParallelConvASREncoder(NeuralModule, Exportable):
+    """
+    Convolutional encoder for ASR models with parallel blocks. CarneliNet can be implemented with this class.
+    """
+
+    def _prepare_for_export(self):
+        m_count = 0
+        for m in self.modules():
+            if isinstance(m, MaskedConv1d):
+                m.use_mask = False
+                m_count += 1
+        logging.warning(f"Turned off {m_count} masked convolutions")
+
+    def input_example(self):
+        """
+        Generates input examples for tracing etc.
+        Returns:
+            A tuple of input examples.
+        """
+        input_example = torch.randn(16, self._feat_in, 256).to(next(self.parameters()).device)
+        return tuple([input_example])
+
+    @property
+    def disabled_deployment_input_names(self):
+        """Implement this method to return a set of input names disabled for export"""
+        return set(["length"])
+
+    @property
+    def disabled_deployment_output_names(self):
+        """Implement this method to return a set of output names disabled for export"""
+        return set(["encoded_lengths"])
+
+    def save_to(self, save_path: str):
+        pass
+
+    @classmethod
+    def restore_from(cls, restore_path: str):
+        pass
+
+    @property
+    def input_types(self):
+        """Returns definitions of module input ports.
+        """
+        return OrderedDict(
+            {
+                "audio_signal": NeuralType(('B', 'D', 'T'), SpectrogramType()),
+                "length": NeuralType(tuple('B'), LengthsType()),
+            }
+        )
+
+    @property
+    def output_types(self):
+        """Returns definitions of module output ports.
+        """
+        return OrderedDict(
+            {
+                "outputs": NeuralType(('B', 'D', 'T'), AcousticEncodedRepresentation()),
+                "encoded_lengths": NeuralType(tuple('B'), LengthsType()),
+            }
+        )
+
+    def __init__(
+        self,
+        jasper,
+        activation: str,
+        feat_in: int,
+        normalization_mode: str = "batch",
+        residual_mode: str = "add",
+        norm_groups: int = -1,
+        conv_mask: bool = True,
+        frame_splicing: int = 1,
+        init_mode: Optional[str] = 'xavier_uniform',
+        aggregation_mode: Optional[str] = None,
+        quantize: bool = False,
+    ):
+        super().__init__()
+        if isinstance(jasper, ListConfig):
+            jasper = OmegaConf.to_container(jasper)
+
+        activation = jasper_activations[activation]()
+        feat_in = feat_in * frame_splicing
+
+        self._feat_in = feat_in
+
+        residual_panes = []
+        encoder_layers = []
+        self.dense_residual = False
+        for lcfg in jasper:
+            dense_res = []
+            if lcfg.get('residual_dense', False):
+                residual_panes.append(feat_in)
+                dense_res = residual_panes
+                self.dense_residual = True
+            groups = lcfg.get('groups', 1)
+            separable = lcfg.get('separable', False)
+            heads = lcfg.get('heads', -1)
+            residual_mode = lcfg.get('residual_mode', residual_mode)
+            se = lcfg.get('se', False)
+            se_reduction_ratio = lcfg.get('se_reduction_ratio', 8)
+            se_context_window = lcfg.get('se_context_size', -1)
+            se_interpolation_mode = lcfg.get('se_interpolation_mode', 'nearest')
+            kernel_size_factor = lcfg.get('kernel_size_factor', 1.0)
+            stride_last = lcfg.get('stride_last', False)
+            aggregation_mode = lcfg.get('aggregation_mode', 'sum')
+            block_dropout = lcfg.get('block_dropout', 0.0)
+            parallel_residual_mode = lcfg.get('parallel_residual_mode', 'sum')
+
+            parallel_blocks = []
+            for kernel_size in lcfg['kernel']:
+                parallel_blocks.append(
+                    JasperBlock(
+                        feat_in,
+                        lcfg['filters'],
+                        repeat=lcfg['repeat'],
+                        kernel_size=[kernel_size],
+                        stride=lcfg['stride'],
+                        dilation=lcfg['dilation'],
+                        dropout=lcfg['dropout'],
+                        residual=lcfg['residual'],
+                        groups=groups,
+                        separable=separable,
+                        heads=heads,
+                        residual_mode=residual_mode,
+                        normalization=normalization_mode,
+                        norm_groups=norm_groups,
+                        activation=activation,
+                        residual_panes=dense_res,
+                        conv_mask=conv_mask,
+                        se=se,
+                        se_reduction_ratio=se_reduction_ratio,
+                        se_context_window=se_context_window,
+                        se_interpolation_mode=se_interpolation_mode,
+                        kernel_size_factor=kernel_size_factor,
+                        stride_last=stride_last,
+                        quantize=quantize,
+                    )
+                )
+            if len(parallel_blocks) == 1:
+                encoder_layers.append(parallel_blocks[0])
+            else:
+                encoder_layers.append(
+                    ParallelBlock(
+                        parallel_blocks,
+                        aggregation_mode=aggregation_mode,
+                        block_dropout_prob=block_dropout,
+                        residual_mode=parallel_residual_mode,
+                        in_filters=feat_in,
+                        out_filters=lcfg['filters'],
+                    )
+                )
+            feat_in = lcfg['filters']
+
+        self._feat_out = feat_in
+
+        self.encoder = torch.nn.Sequential(*encoder_layers)
+        self.apply(lambda x: init_weights(x, mode=init_mode))
+
+    @typecheck()
+    def forward(self, audio_signal, length=None):
+        s_input, length = self.encoder(([audio_signal], length))
+        if length is None:
+            return s_input[-1]
+
+        return s_input[-1], length
+
+
 class ConvASRDecoder(NeuralModule, Exportable):
     """Simple ASR Decoder for use with CTC-based models such as JasperNet and QuartzNet
 
@@ -342,6 +514,86 @@ class ConvASRDecoderClassification(NeuralModule, Exportable):
         return self._num_classes
 
 
+class ECAPAEncoder(NeuralModule, Exportable):
+    """
+    Modified ECAPA Encoder layer without Res2Net module for faster training and inference which achieves
+    better numbers on speaker diarization tasks
+    Reference: ECAPA-TDNN Embeddings for Speaker Diarization (https://arxiv.org/pdf/2104.01466.pdf)
+
+    input:
+        feat_in: input feature shape (mel spec feature shape)
+        filters: list of filter shapes for SE_TDNN modules 
+        kernel_sizes: list of kernel shapes for SE_TDNN modules
+        dilations: list of dilations for group conv se layer
+        scale: scale value to group wider conv channels (deafult:8)
+    
+    output:
+        outputs : encoded output 
+        output_length: masked output lengths
+    """
+
+    @property
+    def input_types(self):
+        """Returns definitions of module input ports.
+        """
+        return OrderedDict(
+            {
+                "audio_signal": NeuralType(('B', 'D', 'T'), SpectrogramType()),
+                "length": NeuralType(tuple('B'), LengthsType()),
+            }
+        )
+
+    @property
+    def output_types(self):
+        """Returns definitions of module output ports.
+        """
+        return OrderedDict(
+            {
+                "outputs": NeuralType(('B', 'D', 'T'), AcousticEncodedRepresentation()),
+                "encoded_lengths": NeuralType(tuple('B'), LengthsType()),
+            }
+        )
+
+    def __init__(
+        self,
+        feat_in: int,
+        filters: list,
+        kernel_sizes: list,
+        dilations: list,
+        scale: int = 8,
+        init_mode: str = 'xavier_uniform',
+    ):
+        super().__init__()
+        self.layers = nn.ModuleList()
+        self.layers.append(TDNNModule(feat_in, filters[0], kernel_size=kernel_sizes[0], dilation=dilations[0]))
+
+        for i in range(len(filters) - 2):
+            self.layers.append(
+                TDNNSEModule(
+                    filters[i],
+                    filters[i + 1],
+                    group_scale=scale,
+                    se_channels=128,
+                    kernel_size=kernel_sizes[i + 1],
+                    dilation=dilations[i + 1],
+                )
+            )
+        self.feature_agg = TDNNModule(filters[-1], filters[-1], kernel_sizes[-1], dilations[-1])
+        self.apply(lambda x: init_weights(x, mode=init_mode))
+
+    def forward(self, audio_signal, length=None):
+        x = audio_signal
+        outputs = []
+
+        for layer in self.layers:
+            x = layer(x, length=length)
+            outputs.append(x)
+
+        x = torch.cat(outputs[1:], dim=1)
+        x = self.feature_agg(x)
+        return x, length
+
+
 class SpeakerDecoder(NeuralModule, Exportable):
     """
     Speaker Decoder creates the final neural layers that maps from the outputs
@@ -351,8 +603,11 @@ class SpeakerDecoder(NeuralModule, Exportable):
         num_classes (int): Number of unique speakers in dataset
         emb_sizes (list) : shapes of intermediate embedding layers (we consider speaker embbeddings from 1st of this layers)
                 Defaults to [1024,1024]
-        pool_mode (str) : Pooling stratergy type. options are 'gram','xvector','superVector'.
-                Defaults to 'xvector'
+        pool_mode (str) : Pooling stratergy type. options are 'xvector','tap', 'attention'
+                Defaults to 'xvector (mean and variance)'
+                tap (temporal average pooling: just mean)
+                attention (attention based pooling)
+
         init_mode (str): Describes how neural network parameters are
             initialized. Options are ['xavier_uniform', 'xavier_normal',
             'kaiming_uniform','kaiming_normal'].
@@ -370,7 +625,12 @@ class SpeakerDecoder(NeuralModule, Exportable):
 
     @property
     def input_types(self):
-        return OrderedDict({"encoder_output": NeuralType(('B', 'D', 'T'), AcousticEncodedRepresentation())})
+        return OrderedDict(
+            {
+                "encoder_output": NeuralType(('B', 'D', 'T'), AcousticEncodedRepresentation()),
+                "length": NeuralType(('B',), LengthsType(), optional=True),
+            }
+        )
 
     @property
     def output_types(self):
@@ -382,35 +642,37 @@ class SpeakerDecoder(NeuralModule, Exportable):
         )
 
     def __init__(
-        self, feat_in, num_classes, emb_sizes=None, pool_mode='xvector', angular=False, init_mode="xavier_uniform",
+        self,
+        feat_in: int,
+        num_classes: int,
+        emb_sizes: Optional[Union[int, list]] = 256,
+        pool_mode: str = 'xvector',
+        angular: bool = False,
+        attention_channels: int = 128,
+        init_mode: str = "xavier_uniform",
     ):
         super().__init__()
         self.angular = angular
         self.emb_id = 2
-        if self.angular:
-            bias = False
-        else:
-            bias = True
+        bias = False if self.angular else True
+        emb_sizes = [emb_sizes] if type(emb_sizes) is int else emb_sizes
 
-        if type(emb_sizes) is str:
-            emb_sizes = emb_sizes.split(',')
-        elif type(emb_sizes) is int:
-            emb_sizes = [emb_sizes]
-        else:
-            emb_sizes = [512, 512]
-
-        self.input_feat_in = feat_in
         self._num_classes = num_classes
-        self._pooling = StatsPoolLayer(feat_in=feat_in, pool_mode=pool_mode)
-        self._feat_in = self._pooling.feat_in
+        self.pool_mode = pool_mode.lower()
+        if self.pool_mode == 'xvector' or self.pool_mode == 'tap':
+            self._pooling = StatsPoolLayer(feat_in=feat_in, pool_mode=self.pool_mode)
+            affine_type = 'linear'
+        elif self.pool_mode == 'attention':
+            self._pooling = AttentivePoolLayer(inp_filters=feat_in, attention_channels=attention_channels)
+            affine_type = 'conv'
 
-        shapes = [self._feat_in]
+        shapes = [self._pooling.feat_in]
         for size in emb_sizes:
             shapes.append(int(size))
 
         emb_layers = []
         for shape_in, shape_out in zip(shapes[:-1], shapes[1:]):
-            layer = self.affineLayer(shape_in, shape_out, learn_mean=False)
+            layer = self.affine_layer(shape_in, shape_out, learn_mean=False, affine_type=affine_type)
             emb_layers.append(layer)
 
         self.emb_layers = nn.ModuleList(emb_layers)
@@ -419,24 +681,34 @@ class SpeakerDecoder(NeuralModule, Exportable):
 
         self.apply(lambda x: init_weights(x, mode=init_mode))
 
-    def affineLayer(self, inp_shape, out_shape, learn_mean=True):
-        layer = nn.Sequential(
-            nn.Linear(inp_shape, out_shape),
-            nn.BatchNorm1d(out_shape, affine=learn_mean, track_running_stats=True),
-            nn.ReLU(),
-        )
+    def affine_layer(
+        self, inp_shape, out_shape, learn_mean=True, affine_type='conv',
+    ):
+        if affine_type == 'conv':
+            layer = nn.Sequential(
+                nn.BatchNorm1d(inp_shape, affine=True, track_running_stats=True),
+                nn.Conv1d(inp_shape, out_shape, kernel_size=1),
+            )
+
+        else:
+            layer = nn.Sequential(
+                nn.Linear(inp_shape, out_shape),
+                nn.BatchNorm1d(out_shape, affine=learn_mean, track_running_stats=True),
+                nn.ReLU(),
+            )
 
         return layer
 
     @typecheck()
-    def forward(self, encoder_output):
-        pool = self._pooling(encoder_output)
+    def forward(self, encoder_output, length=None):
+        pool = self._pooling(encoder_output, length)
         embs = []
 
         for layer in self.emb_layers:
             pool, emb = layer(pool), layer[: self.emb_id](pool)
             embs.append(emb)
 
+        pool = pool.squeeze(-1)
         if self.angular:
             for W in self.final.parameters():
                 W = F.normalize(W, p=2, dim=1)
@@ -444,7 +716,7 @@ class SpeakerDecoder(NeuralModule, Exportable):
 
         out = self.final(pool)
 
-        return out, embs[-1]
+        return out, embs[-1].squeeze(-1)
 
 
 @dataclass
