@@ -132,3 +132,15 @@ class AAYNBaseConfig(MTEncDecModelConfig):
         attn_score_dropout=0.1,
         attn_layer_dropout=0.1,
     )
+
+
+@dataclass
+class MTBottleneckModelConfig(AAYNBaseConfig):
+    model_type: str = 'seq2seq-br'
+    min_logv: float = -6
+    ortho_loss_coef: float = 0.0
+    att_bridge_size: int = 512
+    att_bridge_k: int = 16
+    att_bridge_inner_size: int = 1024
+    non_recon_warmup_batches: int = 200000
+    recon_per_token: bool = True
