@@ -33,6 +33,7 @@ from nemo.collections.nlp.modules.common.megatron.megatron_utils import (
 from nemo.collections.nlp.modules.common.transformer.transformer import NeMoTransformerConfig
 from nemo.collections.nlp.modules.common.transformer.transformer_utils import (
     get_huggingface_transformer,
+    get_megatron_transformer,
     get_nemo_transformer,
 )
 from nemo.utils import logging
@@ -176,4 +177,16 @@ def get_transformer(
             model_name=model_name, pretrained=pretrained, config_dict=config_dict, encoder=encoder
         )
 
+    elif library == 'megatron':
+        model = get_megatron_transformer(
+            model_name=model_name,
+            pretrained=pretrained,
+            config_dict=config_dict,
+            encoder=encoder,
+            checkpoint_file=checkpoint_file,
+        )
+
+    else:
+        raise ValueError("Libary must be 'nemo', 'huggingface' or 'megatron'")
+
     return model
