@@ -12,6 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from collections import OrderedDict
 from typing import List
 
 import torch
@@ -25,7 +26,9 @@ from nemo.collections.asr.data.audio_to_text import AudioToCharWithDursF0Dataset
 from nemo.collections.tts.helpers.helpers import get_mask_from_lengths
 from nemo.collections.tts.models.base import SpectrogramGenerator
 from nemo.collections.tts.modules.talknet import GaussianEmbedding, MaskedInstanceNorm1d, StyleResidual
+from nemo.core import Exportable
 from nemo.core.classes import ModelPT, PretrainedModelInfo, typecheck
+from nemo.core.neural_types import MelSpectrogramType, NeuralType
 
 
 class TalkNetDursModel(ModelPT):
@@ -58,22 +61,30 @@ class TalkNetDursModel(ModelPT):
         durs_pred = pred_durs.exp() - 1
         durs_pred[durs_pred < 0.0] = 0.0
         durs_pred = durs_pred.round().long()
-        acc = ((true_durs == durs_pred) * mask).sum().float() / mask.sum() * 100
 
-        return loss, acc
+        acc = ((true_durs == durs_pred) * mask).sum().float() / mask.sum() * 100
+        acc_dist_1 = (((true_durs - durs_pred).abs() <= 1) * mask).sum().float() / mask.sum() * 100
+        acc_dist_3 = (((true_durs - durs_pred).abs() <= 3) * mask).sum().float() / mask.sum() * 100
+
+        return loss, acc, acc_dist_1, acc_dist_3
 
     def training_step(self, batch, batch_idx):
         _, _, text, text_len, durs, *_ = batch
         pred_durs = self(text=text, text_len=text_len)
-        loss, acc = self._metrics(true_durs=durs, true_text_len=text_len, pred_durs=pred_durs,)
-        train_log = {'train_loss': loss, 'train_acc': acc}
+        loss, acc, acc_dist_1, acc_dist_3 = self._metrics(true_durs=durs, true_text_len=text_len, pred_durs=pred_durs,)
+        train_log = {
+            'train_loss': loss,
+            'train_acc': acc,
+            'train_acc_dist_1': acc_dist_1,
+            'train_acc_dist_3': acc_dist_3,
+        }
         return {'loss': loss, 'progress_bar': train_log, 'log': train_log}
 
     def validation_step(self, batch, batch_idx):
         _, _, text, text_len, durs, *_ = batch
         pred_durs = self(text=text, text_len=text_len)
-        loss, acc = self._metrics(true_durs=durs, true_text_len=text_len, pred_durs=pred_durs,)
-        val_log = {'val_loss': loss, 'val_acc': acc}
+        loss, acc, acc_dist_1, acc_dist_3 = self._metrics(true_durs=durs, true_text_len=text_len, pred_durs=pred_durs,)
+        val_log = {'val_loss': loss, 'val_acc': acc, 'val_acc_dist_1': acc_dist_1, 'val_acc_dist_3': acc_dist_3}
         self.log_dict(val_log, prog_bar=False, on_epoch=True, logger=True, sync_dist=True)
 
     @staticmethod
@@ -178,6 +189,7 @@ class TalkNetPitchModel(ModelPT):
         loss, sil_acc, body_mae = self._metrics(
             true_f0=f0, true_f0_mask=f0_mask, pred_f0_sil=pred_f0_sil, pred_f0_body=pred_f0_body,
         )
+
         val_log = {'val_loss': loss, 'val_sil_acc': sil_acc, 'val_body_mae': body_mae}
         self.log_dict(val_log, prog_bar=False, on_epoch=True, logger=True, sync_dist=True)
 
@@ -223,15 +235,20 @@ class TalkNetPitchModel(ModelPT):
         return list_of_models
 
 
-class TalkNetSpectModel(SpectrogramGenerator):
+class TalkNetSpectModel(SpectrogramGenerator, Exportable):
     """TalkNet's mel spectrogram prediction pipeline."""
 
+    @property
+    def output_types(self):
+        return OrderedDict({"mel-spectrogram": NeuralType(('B', 'D', 'T'), MelSpectrogramType())})
+
     def __init__(self, cfg: DictConfig, trainer: 'Trainer' = None):
         super().__init__(cfg=cfg, trainer=trainer)
         typecheck.set_typecheck_enabled(enabled=False)
 
         cfg = self._cfg
         self.vocab = AudioToCharWithDursF0Dataset.make_vocab(**cfg.train_ds.dataset.vocab)
+        self.blanking = cfg.train_ds.dataset.blanking
         self.preprocessor = instantiate(cfg.preprocessor)
         self.embed = GaussianEmbedding(self.vocab, cfg.d_char)
         self.norm_f0 = MaskedInstanceNorm1d(1)
@@ -297,28 +314,52 @@ class TalkNetSpectModel(SpectrogramGenerator):
     def generate_spectrogram(self, tokens: torch.Tensor, **kwargs) -> torch.Tensor:
         assert hasattr(self, '_durs_model') and hasattr(self, '_pitch_model')
 
-        # Durs
-        text = [
-            AudioToCharWithDursF0Dataset.interleave(
-                x=torch.empty(len(t) + 1, dtype=torch.long, device=t.device).fill_(self.vocab.blank), y=t,
-            )
-            for t in tokens
-        ]
-        text = AudioToCharWithDursF0Dataset.merge(text, value=self.vocab.pad, dtype=torch.long)
-        text_len = torch.tensor(text.shape[-1], dtype=torch.long).unsqueeze(0)
-        durs = self._durs_model(text, text_len)
+        if self.blanking:
+            tokens = [
+                AudioToCharWithDursF0Dataset.interleave(
+                    x=torch.empty(len(t) + 1, dtype=torch.long, device=t.device).fill_(self.vocab.blank), y=t,
+                )
+                for t in tokens
+            ]
+            tokens = AudioToCharWithDursF0Dataset.merge(tokens, value=self.vocab.pad, dtype=torch.long)
+
+        text_len = torch.tensor(tokens.shape[-1], dtype=torch.long).unsqueeze(0)
+        durs = self._durs_model(tokens, text_len)
         durs = durs.exp() - 1
         durs[durs < 0.0] = 0.0
         durs = durs.round().long()
 
         # Pitch
-        f0_sil, f0_body = self._pitch_model(text, text_len, durs)
+        f0_sil, f0_body = self._pitch_model(tokens, text_len, durs)
         sil_mask = f0_sil.sigmoid() > 0.5
         f0 = f0_body * self._pitch_model.f0_std + self._pitch_model.f0_mean
         f0 = (~sil_mask * f0).float()
 
         # Spect
-        mel = self(text, text_len, durs, f0)
+        mel = self(tokens, text_len, durs, f0)
+
+        return mel
+
+    def forward_for_export(self, tokens: torch.Tensor, text_len: torch.Tensor):
+        durs = self._durs_model(tokens, text_len)
+        durs = durs.exp() - 1
+        durs[durs < 0.0] = 0.0
+        durs = durs.round().long()
+
+        # Pitch
+        f0_sil, f0_body = self._pitch_model(tokens, text_len, durs)
+        sil_mask = f0_sil.sigmoid() > 0.5
+        f0 = f0_body * self._pitch_model.f0_std + self._pitch_model.f0_mean
+        f0 = (~sil_mask * f0).float()
+
+        # Spect
+        x, x_len = self.embed(tokens, durs).transpose(1, 2), durs.sum(-1)
+        f0, f0_mask = f0.clone(), f0 > 0.0
+        f0 = self.norm_f0(f0.unsqueeze(1), f0_mask)
+        f0[~f0_mask.unsqueeze(1)] = 0.0
+        x = self.res_f0(x, f0)
+        y, _ = self.model(x, x_len)
+        mel = self.proj(y)
 
         return mel
 
