@@ -14,7 +14,7 @@
 
 
 MAJOR = 1
-MINOR = 1
+MINOR = 2
 PATCH = 0
 PRE_RELEASE = ''
 
@@ -27,7 +27,7 @@ __version__ = '.'.join(map(str, VERSION[:3])) + ''.join(VERSION[3:])
 __package_name__ = 'nemo_toolkit'
 __contact_names__ = 'NVIDIA'
 __contact_emails__ = 'nemo-toolkit@nvidia.com'
-__homepage__ = 'https://docs.nvidia.com/deeplearning/nemo/user-guide/docs/en/main/'
+__homepage__ = 'https://docs.nvidia.com/deeplearning/nemo/user-guide/docs/en/stable/'
 __repository_url__ = 'https://github.com/nvidia/nemo'
 __download_url__ = 'https://github.com/NVIDIA/NeMo/releases'
 __description__ = 'NeMo - a toolkit for Conversational AI'
